/* $Header: /Moxie Technologies/Ikon/ikonpci/Source/IKONCTL.H 9     5/01/97 11:38a Polfer $ */
/******************************************************************************
**
** Copyright 1996 Moxie Technologies, Inc.,  All Rights Reserved.
**
** This software contains the valuable trade secrets of Moxie Technologies, Inc.
** ("Moxie").  The software is protected under copyright laws as an unpublished
** work of Moxie.  Notice is for informational purposes only and does not imply
** publication.  The user of this software may make copies of the software for
** use with parts manufactured by Moxie or under license from Moxie and for no
** other use.
**
******************************************************************************/

/******************************************************************************
**
**	Module Name:	ikonctl.h
**
**	Description:	Public definitions used to interface to the IKONPCI.SYS
**					parallel class driver through DeviceIoControl.  These
**					definitions closely follow the IOCTL definitions used in
**					previous workstation versions of Ikon drivers.  See the
**					technical reference guide for more information on the
**					use of these control codes.
**
**	Programmers:	Daniel Polfer (dap)
**
**	History:		02/17/96 (dap)  Initial creation.
**					04/14/97 (dap)	Modified to support burst mode.
**
**	Notes:			This file created using 4 spaces per tab.
**
******************************************************************************/

#ifndef __IKONCTL_H__
#define __IKONCTL_H__

/*
** ----------------------------------------------------------------------------
** General macros.
** ----------------------------------------------------------------------------
*/

#define DEFAULT_TIMEOUT         300             // The default timeout in secs.
#define IKONPCI_DEVICENAME      L"\\Device\\Ikonpci"

/*
** ----------------------------------------------------------------------------
** Macros used to manage the drivers "Parameters" key in the registry.  The
** REGVAL_ items below define the name of a parameter, and the REGDFLT_ macro
** specifies the default value if there isn't a Parameters subkey or there is
** no value in the subkey.
**
** Do NOT use the BreakOnEntry value without a kernel debugger active or you
** will get the blue screen of death.  Also, certain settings of DebugLevel
** can cause an excessive number of events to be logged by the driver which
** can effect performance.  Use both parameters with care!
** ----------------------------------------------------------------------------
*/

#define REGKEY_PARAMETERS       L"\\Parameters"

#define REGVAL_DEBUGLEVEL       L"DebugLevel"
#define REGDFLT_DEBUGLEVEL      0

#define REGVAL_BREAKONENTRY     L"BreakOnEntry"
#define REGDFLT_BREAKONENTRY    0

#define REGVAL_VPIDOSNAME       L"VPIDosName"
#define REGDFLT_VPIDOSNAME      L"VPT"

#define REGVAL_CENTDOSNAME      L"CENTDosName"
#define REGDFLT_CENTDOSNAME     L"CPT"

#define REGVAL_PORTSPEED        L"Portspeed"
#define REGDFLT_PORTSPEED       1

/*
** ----------------------------------------------------------------------------
**                       TRADITIONAL IKON CONTROL CODES
** ----------------------------------------------------------------------------
**
** Manifest constants for DeviceIoControl calls to the IKONPCI.SYS driver.
** These constants mirror the original Ikon specific codes used by Unix drivers.
** The DeviceIoControl function sends a control code directly to the parallel
** class driver, causing the specified Ikon board to perform the specified
** operation.  It is structured as follows:
**
**     BOOL DeviceIoControl
**     (
**         HANDLE       hDevice,            // handle to Ikon 10115/10117 
**         DWORD        dwIoControlCode,	// control code of operation
**         LPVOID       lpInBuffer,     	// pointer to input buffer
**         DWORD        nInBufferSize,      // size of input buffer
**         LPVOID       lpOutBuffer,	    // pointer to output buffer
**         DWORD        nOutBufferSize,	    // size of output buffer
**         LPDWORD      lpBytesReturned,	// pointer to output byte count
**         LPOVERLAPPED lpOverlapped 	    // pointer to overlapped structure
**     );                                   // for asynchronous operation
**
** The following dwIoControlCode's are supported by the IKONPCI.SYS driver:
**
**     dwIoControlCode      DIR Description
**     -------------------  --- -----------------------------------------------
**     IHCPIO_DEV_RESET	    N/A reset attached device
**     IHCPIO_SET_CONFIG	IN  select device timing and busy mode
**     IHCPIO_SET_DMATIME	IN  set dma timeout in seconds (not implemented)
**     IHCPIO_SET_FIFOTIME  IN  set fifo<1/2 full and fifo empty+device timeout
**     IHCPIO_GET_STATUS	OUT return device status
**     IHCPIO_GET_BOARD	    OUT return board type
**     IHCPIO_SET_VMODE	    IN  set versatec print/plot/spp mode
**     IHCPIO_SET_VMODEX	IN  same as VMODE, present for back compatibility
**     IHCPIO_SET_AUTOLTR   IN  set auto-LTR byte width or 0 to disable
**     IHCPIO_V_CMD		    IN  send pulse command to plotter (versatec only)
**     IHCPIO_DATA_OUT		IN  send data bytes from input buffer to device
**     IHCPIO_RDY_WAIT		N/A wait for fifo empty and device ready
**     IHCPIO_HALF_WAIT	    N/A wait for fifo less than 1/2 full
**     IHCPIO_STREAM_ON	    N/A select data streaming mode (centronics only)
**     IHCPIO_STREAM_OFF	N/A clear data streaming mode (centronics only)
**     IHCPIO_GET_FLAGS	    OUT returns unit_flags
**     IHCPIO_GET_FIFO		OUT returns fifo status
**     IHCPIO_SOFT_ACK		N/A simulates ack sequence from device
**     IHCPIO_MASTER_CLEAR  N/A resets board and clears fifo (FACTORY USE ONLY)
**     IHCPIO_GET_REGS		OUT return board's registers in array of 8 DWORDS
**     
** In the above table, N/A indicates that lpInBuffer/nInBufferSize/lpOutBuffer/
** nOutBufferSize do not matter and will be ignored.  For all but
** the IHCPIO_GET_REGS command, lpInBuffer should point to a DWORD and 
** nInBufferSize should be 4.  Similarly, for all bu IHCPIO_DATA_OUT,
** lpOutBuffer should point to a DWORD and nOutBufferSize should be 4.  The
** IHCPIO_GET_REGS command requires that lpInBuffer point to an array of 8
** DWORDS and nInBufferSize should be set to 32 bytes or more.  IHCPIO_DATA_OUT
** is present for legacy code, and lpInBuffer should point to an array of up
** to 255 bytes while nInBufferSize should be a value from 0x00-0xFF for the
** number of bytes to be sent out the port manually.  See the comments below
** or the technical reference manual for more information on the use of these
** commands.
** ----------------------------------------------------------------------------
*/

#define IHCPIO_DEV_RESET        CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x800,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IHCPIO_SET_CONFIG       CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x808,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define     IHCPIO_SPEED0       0x00000000	/* slowest device speed	*/
#define     IHCPIO_SPEED1	    0x00000004
#define     IHCPIO_SPEED2	    0x00000008
#define     IHCPIO_SPEED3	    0x0000000C	/* fastest device timing */
#define     IHCPIO_IGNORE_BUSY  0x00000080	/* ignore centr busy */
#define     IHCPIO_USE_BUSY     0x00000100  /* use BUSY not ACK */
#define     IHCPIO_FOUR_EDGE    0x00000200  /* use four edge handshakes */
#define 	IHCPIO_BURST		0x00000400  /* burst data in blocks */

#define IHCPIO_SET_DMATIME      CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x810,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IHCPIO_SET_FIFOTIME     CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x818,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IHCPIO_GET_STATUS       CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x820,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define     IHCPIO_DEV_RDY		0x00000010	/* v-rdy or centr ack asserted */
#define     IHCPIO_DEV_BUSY		0x00000008	/* centronics asserting busy */
#define     IHCPIO_DEV_FAULT 	0x00000004	/* centronics asserting fault */
#define     IHCPIO_DEV_POUT		0x00000002	/* v or c no paper */
#define     IHCPIO_DEV_SEL		0x00000001	/* v online or c selected */

#define IHCPIO_GET_BOARD        CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x828,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define 	IHCPIO_VERS_DIFF    0x000000A0	/* board strapped for VPI DIFF */
#define 	IHCPIO_VERS_TTL     0x00000060	/* board strapped for VPI TTL */
#define 	IHCPIO_CENT         0x000000C0	/* board strapped for CENTRONICS */

#define IHCPIO_SET_VMODE        CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x830,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IHCPIO_SET_VMODEX       CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x838,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define     IHCPIO_V_SPP		0x00000002	/* sets spp w/MODE(X) */
#define     IHCPIO_V_PLOT		0x00000001	/* sets plot mode w/MODE(X) */
#define     IHCPIO_V_NPRINT		0x00000000	/* 0 = normal print mode */

#define IHCPIO_SET_AUTOLTR		CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x83C,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IHCPIO_V_CMD            CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x840,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define     IHCPIO_VCLR			0x00000003	/* remote buffer clear */
#define     IHCPIO_VFED			0x00000002	/* remote form feed */
#define     IHCPIO_VEOT			0x00000001	/* remote eot */
#define     IHCPIO_VLTR			0x00000000	/* remote line terminate*/

#define IHCPIO_DATA_OUT         CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x848,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IHCPIO_RDY_WAIT         CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x850,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IHCPIO_HALF_WAIT        CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x858,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IHCPIO_STREAM_ON        CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x860,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IHCPIO_STREAM_OFF       CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x868,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IHCPIO_GET_FLAGS        CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x870,METHOD_BUFFERED,FILE_ANY_ACCESS)

#define IHCPIO_GET_FIFO         CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x878,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define     IHCPIO_FIFO_FULL	0x00000010	/* set if fifo full */
#define     IHCPIO_FIFO_HALF 	0x00000008	/* if half full */
#define     IHCPIO_FIFO_EMPTY 	0x00000004	/* if empty */

#define IHCPIO_SOFT_ACK         CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x880,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IHCPIO_MASTER_CLEAR     CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x888,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IHCPIO_GET_REGS         CTL_CODE(FILE_DEVICE_PARALLEL_PORT,0x890,METHOD_BUFFERED,FILE_ANY_ACCESS)
                                            /* dwRegs[0] = INTERRUPT MASK */
                                            /* dwRegs[1] = MODE */
                                            /* dwRegs[2] = DEVICE CONTROL */
                                            /* dwRegs[3] = INTERFACE CONTROL */
                                            /* dwRegs[4] = INTERFACE STATUS */
                                            /* dwRegs[5] = DEVICE STATUS */
                                            /* dwRegs[6] = AUTO LTR COUNT LOW */
                                            /* dwRegs[7] = AUTO LTR COUNT HIGH */

#endif //__IKONCTL_H__
