/*
 * ikonex.c
 * hardcopy driver/board test and exercise program
 * based on IKONdma.c v2.7 by Interface Consultants
 *
 * this program outputs files in print or plot mode
 * and allows issuing assorted commands to the board,
 *
 * this version modified to compile and run on windows
 * the code is an ugly mash-up of windows and unix-
 * flavors.  sorry about that.
 * 
 * Tahoma Technology (formerly Ikon Corporation)
 * 2819 Elliott Ave
 * Suite 203
 * Seattle, WA, USA 98121
 *
 * 206.728.6465
 * http://www.tahomatech.com
 * tahoma@tahomatech.com
 * 
 *
 * 1 Nov, 2000
 * 
 * 10 Feb, 2001		define _IOC_SIZEMASK & _IOC_SIZESHIFT for Solaris
 *
 * 19 Nov, 2001		corrected errors in self documenting printfs
 *					made "Tahoma" changes in comments
 *
 * 6 May, 2002		changed include file from ihcp_io.h to ikonex.h
 *					added crlf and ff (ascii) commands
 *					updated company info
 *
 * 14 June, 2002	changed back to including ihcp_io.h
 *
 * 8 July, 2002		corrected mode self doc printf
 *
 * 20 April, 2004	hack to try to make interactive
 *
 * 8 March, 2007	add endian ioctl support
 * 					move IkonRegStruct here and eliminate .h file
 *
 * 13 March, 2012	rework for windows - this version for windows
 *					only, see elsewhere for *nix version
 *
 *					will call the moxie pci driver as modded by oyo
 *
 *					no getopt in visual c++, so use our getcommand
 *					and friends
 *
 *					convert some variables to ugly windows types
 *
 *					modify ioctl command list to match Moxie driver
 *
 * 3 May, 2012		add O_BINARY flag to file open
 */

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <io.h>

#include "./ikonctl.h"

 /*
  * banner, version, etc, displayed at prog start
  */

static char IkonexBanner[] = {
	"ikonex for windows"
};

static char VersionString[] = {
	"3 May, 2012"
};

static char TahomaTech[] = {
	"Tahoma Technology (formerly Ikon Corporation), Seattle, WA, USA"
};

static char ContactInfo[] = {
	"www.tahomatech.com"
};

#define TRUE 1
#define FALSE 0

/*
 * we need a local buffer for file transfer, etc
 * use malloc to get the storage - malloc guarantees
 * alignment suitable for any type of object
 * we need 32-bit alignment for WriteFile
 */

#define LOCBUFFSIZE 10000

static char *local_buff_p;

/*
 * global variables for device i/o calls
 */

static HANDLE dev_handle = INVALID_HANDLE_VALUE;
static DWORD cmd_arg;
static DWORD byte_count;

/*
 * Moxie driver only returns 8 registers
 */

struct IkonRegStruct {
	DWORD InterruptMask;
	DWORD Mode;
	DWORD DeviceControl;
	DWORD InterfaceControl;
	DWORD InterfaceStatus;
	DWORD DeviceStatus;
	DWORD AutoLtrCountLow;
	DWORD AutoLtrCountHigh;
} IkonReg;

/*
 * utility variables
 */

static int interactive;
static int verbose;
static int arg_set = 0;
static int file_ptr = -1;


/*
 * global stuff for our simple getopt implementation
 * init optind to 1st actual cmd line arg (if any) -
 * index 0 is program name as called, index 1 is 1st arg
 */

static char *optarg;
static int optind = 1;
static int opterr = 0;
static int optopt;


/*
 * local arg string when interactive input used
 * and local arg ptr used by interactive and cmd line
 */

static char loptstr[132];
static char *loptarg;

/*
 * close handle if open
 */

void close_handle(void)
{
	if(dev_handle != INVALID_HANDLE_VALUE)
		CloseHandle(dev_handle);
	dev_handle = INVALID_HANDLE_VALUE;
}

/*
 * only exit if not interactive
 */

void try_exit(void)
{
	if(interactive)
		return;
	close_handle();
	exit(-1);
}

/*
 * the microsoft i/o functions are so butt-ugly we will try
 * to wrap them in unix-like functions and see how that looks
 *
 * dev_open() opens tahoma device
 * dev_ioctl() tahoma device ioctl - no device i/o
 * dev_ioctl_out() tahoma device ioctl - cmd/data to device
 * dev_ioctl_in() tahoma device ioctl - status/data from device
 * dev_write() tahoma device write - data block to device
 *
 * dev_write() expects the incoming buffer to be 32-bit aligned
 *
 * functions return success (0) or failure (!0) and may exit directly
 * in error case if not interactive mode - some parameters
 * are global, not passed as args
 *
 * note that ms in and out directions are reversed from real
 * life - when deviceiocontrol says OUT it means FROM device
 * go figure (when I say out, I mean it...)
 */

int dev_open(char *dev_path)
{
    dev_handle = CreateFile(dev_path, GENERIC_WRITE, 0,
		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    if(dev_handle == INVALID_HANDLE_VALUE)
    {
		printf("Error (dev_open): CreateFile returns %d opening device \"%s\"\n",
           GetLastError(), dev_path);
        try_exit();
		return -1;
    }

	return 0;
}

int dev_ioctl(DWORD command)
{
   if(!DeviceIoControl(dev_handle, command,
		NULL, 0, NULL, 0, &byte_count, NULL))
	{
		printf("Error (dev_ioctl): DeviceIoControl returns %d\n",
			GetLastError());
		try_exit();
		return -1;
	}

	return 0;
}

int dev_ioctl_out(DWORD command, LPVOID arg_p, DWORD arg_size)
{
	if(!DeviceIoControl(dev_handle, command,
		arg_p, arg_size, NULL, 0, &byte_count, NULL))
	{
		printf("Error (dev_ioctl_out): DeviceIoControl returns %d\n",
			GetLastError());
		try_exit();
		return -1;
	}

	return 0;
}

int dev_ioctl_in(DWORD command, LPVOID arg_p, DWORD arg_size)
{
	if(!DeviceIoControl(dev_handle, command,
		NULL, 0, arg_p, arg_size, &byte_count, NULL))
	{
		printf("Error (dev_ioctl_in): DeviceIoControl returns %d\n",
			GetLastError());
		try_exit();
		return -1;
	}

	return 0;
}

int dev_write(LPVOID buff_p, DWORD buff_size)
{
	if(!WriteFile(dev_handle, buff_p, buff_size, &byte_count, NULL))
	{
		printf("Error (dev_write): WriteFile returns %d\n",
			GetLastError());
		try_exit();
		return -1;
	}
	return 0;
}

/*
 * simple replacement for getopt fcn for use in win apps where
 * unix getopt not available
 *
 * return single char option if match, '?' if no match, EOF if
 * arg list exhausted, and *optarg set to option arg if used
 * if option requires argument, but list exhausted, '?' returned
 * unless 1st char of opt string is ':', in which case ':'
 * returned
 *
 * this simplified version only allows one option per "-"
 * multiple options require "-x -y ... -x" form
 * options w/args may use either "-x arg" or "-xarg" form
 *
 * on exit, optind points to next argv element
 *
 * global optind MUST be initialized to 1!
 */

static int getcmdopt(int argc, char * const argv[], const char *optstring)
{
	char *curarg;
	int missingarg;

	if(optind >= argc)
		return EOF;

	curarg = argv[optind];
	if(*curarg != '-')
	{
		optarg = NULL;
		return EOF;
	}
	curarg++;
	optopt = *curarg;
	missingarg = (*optstring == ':') ? ':' : '?';
	while(*optstring) {
		if(*optstring == *curarg)
		{
			/* option match */
			if(*(optstring + 1) == ':')
			{
				/*
				 * arg required
				 * if next char in curarg not null
				 * return ptr to _that_ as arg
				 * otherwise use next argv
				 */

				if(*(curarg + 1) != '\0')
				{
					optarg = curarg + 1;
					optind++;
					return *curarg;
				}
				if(++optind >= argc)
				{
					optarg = NULL;
					return missingarg;
				}
				optarg = argv[optind];
				optind++;
				return *curarg;
			}
			optarg = NULL;
			optind++;
			return *curarg;
		}
		optstring++;
	}
	optopt = *curarg;
	return '?';
}


/*
 * get next option (and possibly an argument) from interactive
 * input
 *
 * make it look like cmd line getcmdopt to extent possible
 *
 * only one option per line allowed
 */

static int getinteropt(char *optstring)
{
	char linestr[132];
	char cstr[8];
	char *lineptr;
	int c;
	int missingarg;

	/*
	 * scan for the option flag and argument
	 * avoid any leading '-' and if s (string)
	 * scan for characters between double quotes
	 */

	printf("enter command (h = help, q = quit): ");
	strcpy(linestr,"?");
	strcpy(cstr,"?");
	strcpy(loptstr,"");
	fgets(linestr, 130, stdin);
	if((lineptr = strchr(linestr, '-')) != NULL)
		lineptr++;
	else
		lineptr = linestr;
	sscanf(lineptr, "%1s %130s", cstr, loptstr);
	c = cstr[0];
	optopt = c;
	missingarg = (*optstring == ':') ? ':' : '?';
	while(*optstring != c)
	{
		if(*optstring == '\0')
		{
			/* no match */
			loptarg = NULL;
			return '?';
		}
		optstring++;
	}

	/*
	 *optstring points at matching option
	 * test for '"' in input string
	 * if so, arg is all within quotes
	 * if not, arg already in loptstr
	 */

	if(*(optstring + 1) == ':')
	{
		if(strchr(lineptr, '"'))
			sscanf(lineptr, "%*1s \"%[^\"]", loptstr);
		loptarg = loptstr;
		/* test for missing arg */
		if(!strlen(loptarg))
		{
			loptarg = NULL;
			return missingarg;
		}
	}
	return c;
}


/*
 * get next option and arg from command line or input stream
 *
 * return from getcmdopt or getinteropt passed back to caller:
 * if unknown option, return '?', if missing arg, return ':'
 * (if opstring begins w/':'), if no further options, return EOF
 *
 * if option arg used, loptarg point to arg string
 */

static int getcommand(int argc, char * const argv[], char *optstring)
{
	int c;

	if(interactive)
	{
		c = getinteropt(optstring);
		/* getinteropt sets loptarg */
	}
	else
	{

		c = getcmdopt(argc, argv, optstring);
		loptarg = optarg;
	}
	if(verbose)
		printf("(getcommand) option = \"%c\", arg = \"%s\"\n",
			(c != EOF) ? c : ' ', loptarg ? loptarg : "(NULL)");
	return c;
}

/*
 * send a file to the device using WriteFile
 * return on error - handled elsewhere
 */

int send_file(char *file_name)
{
	int file_handle = -1;
	int bytes_read;

	if(dev_handle == INVALID_HANDLE_VALUE)
	{
		printf("Error (send_file): device not open - use -w<> to open device\n");
		return -1;
	}

	/*
	 * open the file (this OK in MS C?)
	 */

	file_handle = open(file_name, O_RDONLY | O_BINARY);
	if (file_handle == -1) {
		printf("Error (send_file): unable to open file: \"%s\"\n", file_name);
		return -1;
	}

	/*
	 * simple output loop
	*/

	while(1)
	{
		bytes_read = read(file_handle, local_buff_p, LOCBUFFSIZE);
		if(bytes_read == 0)
			break;
		if(bytes_read == -1) {
			printf("Error (send_file): error reading file \"%s\"\n", file_name);
			close(file_handle);
			return -1;
		}
		if(dev_write(local_buff_p, bytes_read))
		{
			printf("Error (send_file): dev_write returns error)\n");
			close(file_handle);
			return -1;
		}
	}
	close(file_handle);
	return 0;
}

/*
 * send command via ioctl
 */

int issue_cmd(char *cmd_str)
{
	DWORD local_arg;

	if (verbose)
		printf("issue_cmd cmd_str = %s\n", cmd_str);

	/*
	 * if error, just return -1
	 * higher level code will do try_exit()
	 */

	if (dev_handle == INVALID_HANDLE_VALUE)
	{
		printf("Error (issue_cmd): device not open - use -w<> to open device\n");
		return 1;
	}

	/*
	 * SPEED COMMANDS
	 */

	if(!strcmp(cmd_str, "SPEED_0"))
	{
		if(verbose)
			printf("cmd = SPEED_0\n");
		local_arg = IHCPIO_SPEED0;
		if(dev_ioctl_out(IHCPIO_SET_CONFIG, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(SPEED_0) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "SPEED_1"))
	{
		if(verbose)
			printf("cmd = SPEED_1\n");
		local_arg = IHCPIO_SPEED1;
		if(dev_ioctl_out(IHCPIO_SET_CONFIG, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(SPEED_1) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "SPEED_2"))
	{
		if(verbose)
			printf("cmd = SPEED_2\n");
		local_arg = IHCPIO_SPEED2;
		if(dev_ioctl_out(IHCPIO_SET_CONFIG, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(SPEED_2) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "SPEED_3"))
	{
		if(verbose)
			printf("cmd = SPEED_3\n");
		local_arg = IHCPIO_SPEED3;
		if(dev_ioctl_out(IHCPIO_SET_CONFIG, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(SPEED_3) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * MODE COMMANDS
	 */

	if(!strcmp(cmd_str, "VPRINT"))
	{
		if(verbose)
			printf("cmd = VPRINT\n");
		local_arg = IHCPIO_V_NPRINT;;
		if(dev_ioctl_out(IHCPIO_SET_VMODE, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(VPRINT) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "VPLOT"))
	{
		if(verbose)
			printf("cmd = VPLOT\n");
		local_arg = IHCPIO_V_PLOT;
		if(dev_ioctl_out(IHCPIO_SET_VMODE, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(VPLOT) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "VPRINTPLOT"))
	{
		if(verbose)
			printf("cmd = VPRINTPLOT\n");
		local_arg = IHCPIO_V_SPP;
		if(dev_ioctl_out(IHCPIO_SET_VMODE, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(VPRINTPLOT) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "DEV_RESET"))
	{
		if(verbose)
			printf("cmd = DEV_RESET\n");
		if(dev_ioctl(IHCPIO_DEV_RESET))
		{
			printf("Error (issue_cmd): dev_ioctl(DEV_RESET) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * PULSED COMMANDS
	 */

	if(!strcmp(cmd_str, "VCLR"))
	{
		if(verbose)
			printf("cmd = VCLR\n");
		local_arg = IHCPIO_VCLR;
		if(dev_ioctl_out(IHCPIO_V_CMD, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(VCLR) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "VFED"))
	{
		if(verbose)
			printf("cmd = VFED\n");
		local_arg = IHCPIO_VFED;
		if(dev_ioctl_out(IHCPIO_V_CMD, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(VFED) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "VEOT"))
	{
		if(verbose)
			printf("cmd = VEOT\n");
		local_arg = IHCPIO_VEOT;
		if(dev_ioctl_out(IHCPIO_V_CMD, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(VEOT) returns error\n");
			return -1;
		}
		return 0;
	}

	if(!strcmp(cmd_str, "VLTR"))
	{
		if(verbose)
			printf("cmd = VLTR\n");
		local_arg = IHCPIO_VLTR;
		if(dev_ioctl_out(IHCPIO_V_CMD, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(VLTR) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * GET REGISTERS - Moxie driver only returns 8
	 */

	if(!strcmp(cmd_str, "GET_REGS"))
	{
		if(verbose)
			printf("cmd = GET_REGS\n");
		if(dev_ioctl_in(IHCPIO_GET_REGS, &IkonReg, sizeof(IkonReg)))
		{
			printf("Error (issue_cmd): dev_ioctl_in(GET_REGS) returns error\n");
			return -1;
		}

		printf("\n");
		printf("GET_REGS - Interrupt Mask      = 0x%x\n", IkonReg.InterruptMask & 0xFF);
		printf("GET_REGS - Mode                = 0x%x\n", IkonReg.Mode & 0xFF);
		printf("GET_REGS - Device Control      = 0x%x\n", IkonReg.DeviceControl & 0xFF);
		printf("GET_REGS - Interface Control   = 0x%x\n", IkonReg.InterfaceControl & 0xFF);
		printf("GET_REGS - Interface Status    = 0x%x\n", IkonReg.InterfaceStatus & 0xFF);
		printf("GET_REGS - Device Status       = 0x%x\n", IkonReg.DeviceStatus & 0xFF);
		printf("GET_REGS - Auto Ltr Count Low  = 0x%x\n", IkonReg.AutoLtrCountLow & 0xFF);
		printf("GET_REGS - Auto Ltr Count High = 0x%x\n", IkonReg.AutoLtrCountHigh & 0xff);
		printf("\n");

		return 0;
	}

	/*
	 * GET STATUS - formatted
	 */

	if(!strcmp(cmd_str, "GET_STATUS"))
	{
		if(verbose)
			printf("cmd = GET_STATUS\n");
		if(dev_ioctl_in(IHCPIO_GET_STATUS, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_in(GET_STATUS) returns error\n");
			return -1;
		}

		printf("GET_STATUS - status = 0x%x\n", local_arg);
		if(local_arg & IHCPIO_DEV_RDY)
			printf("GET_STATUS - Device Ready\n");
		if(local_arg & IHCPIO_DEV_BUSY)
			printf("GET_STATUS - Device Busy\n");
		if(local_arg & IHCPIO_DEV_FAULT)
			printf("GET_STATUS - Device Fault\n");
		if(local_arg & IHCPIO_DEV_POUT)
			printf("GET_STATUS - Device Paper Out\n");
		if(local_arg & IHCPIO_DEV_SEL)
			printf("GET_STATUS - Device Online\n");

		return 0;
	}

	/*
	 * GET BOARD
	 */

	if(!strcmp(cmd_str, "GET_BOARD"))
	{
		if(verbose)
			printf("cmd = GET_BOARD\n");
		if(dev_ioctl_in(IHCPIO_GET_BOARD, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_in(GET_BOARD) returns error\n");
			return -1;
		}

		printf("GET_BOARD - Board = 0x%x\n", local_arg);
		if(local_arg == IHCPIO_VERS_DIFF)
			printf("GET_BOARD - Strapping is Versatec Differential\n");
		if(local_arg == IHCPIO_VERS_TTL)
			printf("GET_BOARD - Strapping is Versatec TTL\n");
		if(local_arg == IHCPIO_CENT)
			printf("GET_BOARD - Strapping is Centronics\n");

		return 0;
	}

	/*
	 * GET FLAGS
	 */

	if(!strcmp(cmd_str, "GET_FLAGS"))
	{
		if(verbose)
			printf("cmd = GET_FLAGS\n");
		if(dev_ioctl_in(IHCPIO_GET_FLAGS, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_in(GET_FLAGS) returns error\n");
			return -1;
		}
		printf("GET_FLAGS - Flags = 0x%x\n", local_arg);
		return 0;
	}

	/*
	 * GET FIFO
	 */

	if(!strcmp(cmd_str, "GET_FIFO"))
	{
		if(verbose)
			printf("cmd = GET_FIFO\n");
		if(dev_ioctl_in(IHCPIO_GET_FIFO, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_in(GET_FIFO) returns error\n");
			return -1;
		}

		printf("GET_FIFO - FIFO = 0x%x\n", local_arg);
		if(local_arg & IHCPIO_FIFO_FULL)
			printf("GET_FIFO - FIFO Full\n");
		if(local_arg & IHCPIO_FIFO_HALF)
			printf("GET_FIFO - FIFO Half Full\n");
		if(local_arg & IHCPIO_FIFO_EMPTY)
			printf("GET_FIFO - FIFO Empty\n");

		return 0;
	}

	/*
	 * DEVICE STATUS - unformatted
	 */

	if(!strcmp(cmd_str, "DEV_STATUS"))
	{
		if(verbose)
			printf("cmd = DEV_STATUS\n");
		if(dev_ioctl_in(IHCPIO_GET_STATUS, &local_arg, sizeof(local_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_in(DEV_STATUS) returns error\n");
			return -1;
		}
		printf("DEV_STATUS - Status = 0x%x\n", local_arg);
		return 0;
	}

	/*
	 * READY WAIT
	 */

	if(!strcmp(cmd_str, "RDY_WAIT"))
	{
		if(verbose)
			printf("cmd = RDY_WAIT\n");
		if(dev_ioctl(IHCPIO_RDY_WAIT))
		{
			printf("Error (issue_cmd): dev_ioctl(RDY_WAIT) returns error\n");
			return -1;
		}
		printf("RDY_WAIT - wait complete\n");
		return 0;
	}

	/*
	 * HALF WAIT
	 */

	if(!strcmp(cmd_str, "HALF_WAIT"))
	{
		if(verbose)
			printf("cmd = HALF_WAIT\n");
		if(dev_ioctl(IHCPIO_HALF_WAIT))
		{
			printf("Error (issue_cmd): dev_ioctl(HALF_WAIT) returns error\n");
			return -1;
		}
		printf("HALF_WAIT - wait complete\n");
		return 0;
	}

	/*
	 * STREAM ON
	 */

	if(!strcmp(cmd_str, "STREAM_ON"))
	{
		if(verbose)
			printf("cmd = STREAM_ON\n");
		if(dev_ioctl(IHCPIO_STREAM_ON))
		{
			printf("Error (issue_cmd): dev_ioctl(STREAM_ON) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * STREAM OFF
	 */

	if(!strcmp(cmd_str, "STREAM_OFF"))
	{
		if(verbose)
			printf("cmd = STREAM_OFF\n");
		if(dev_ioctl(IHCPIO_STREAM_OFF))
		{
			printf("Error (issue_cmd): dev_ioctl(STREAM_OFF) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * MASTER CLEAR
	 */

	if(!strcmp(cmd_str, "MASTER_CLEAR"))
	{
		if(verbose)
			printf("cmd = MASTER_CLEAR\n");
		if(dev_ioctl(IHCPIO_MASTER_CLEAR))
		{
			printf("Error (issue_cmd): dev_ioctl(MASTER_CLEAR) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * SOFT ACK
	 */

	if(!strcmp(cmd_str, "SOFT_ACK"))
	{
		if(verbose)
			printf("cmd = SOFT_ACK\n");
		if(dev_ioctl(IHCPIO_SOFT_ACK))
		{
			printf("Error (issue_cmd): dev_ioctl(SOFT_ACK) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * SET CONFIG
	 */

	if(!strcmp(cmd_str, "SET_CONFIG"))
	{
		if(verbose)
			printf("cmd = SET_CONFIG\n");
		if(!arg_set)
		{
			printf("Error (issue_cmd): SET_CONFIG requires argument\n");
			return -1;
		}
		if(dev_ioctl_out(IHCPIO_SET_CONFIG, &cmd_arg, sizeof(cmd_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(SET_CONFIG) returns error\n");
			return -1;
		}
		return 0;
	}

	/*
	 * AUTO LTR COUNT -setting enables, setting to 0 disables
	 */

	if(!strcmp(cmd_str, "SET_AUTOLTR"))
	{
		if(verbose)
			printf("cmd = SET_AUTOLTR\n");
		if(!arg_set)
		{
			printf("Error (issue_cmd): SET_AUTOLTR requires argument\n");
			return -1;
		}
		if(dev_ioctl_out(IHCPIO_SET_AUTOLTR, &cmd_arg, sizeof(cmd_arg)))
		{
			printf("Error (issue_cmd): dev_ioctl_out(SET_AUTOLTR) returns error\n");
			return -1;
		}
		return 0;
	}

	printf("Error (issue_cmd): invalid command: \"%s\"\n", cmd_str);
	return -1;
}

/*
 * print help screen
 */

void useage(void)
{
	char dummystr[8];

	/*
	 * print help screen(s)
	 * note \\\\.\\ in printf will show as \\.\ when printed,
	 * which is the actual path: \\.vpt1 for instance
	 */
	
	printf("\nUsage: ikonex -w<> [-m<>] [-f<>]  [-b<>] [-a<>] [-t<>] [-c<>] [-s<>]\n");
	printf(" 	              [-n] [-p] [-v] [-i] [-h] [-q]\n");
	printf("        -w<output device> (required - e.g. \\\\.\\vpt1 or \\\\.\\cpt1\n");
	printf("        -m<mode> mode for transfer, PRINT or PLOT\n");
	printf("        -f<filename> send file to device\n");
	printf("        -b<byte> send single hex byte\n");
	printf("        -a<argument> hex argument required (*) for some commands\n");
	printf("        -t<timeout> set FIFO timeout in seconds\n");
	printf("        -c<command> remote command to board/plotter, can be:\n");
	printf(" \n");
	printf("                 SPEED_0 - slowest handshake speed\n");
	printf("                 SPEED_1\n");
	printf("                 SPEED_2\n");
	printf("                 SPEED_3 - fastest handshake speed\n");
	printf("                 VPRINT - set Versatec print mode\n");
	printf("                 VPLOT - set Versatec plot mode\n");
	printf("\n--More-- (enter)\n");
	fgets(dummystr, 2, stdin);
	printf("                 VPRINTPLOT - set Versatec simultaneous print/plot mode\n");
	printf("                 DEV_RESET - send reset pulse to plotter\n");
	printf("                 VCLR - Versatec Remote Clear\n");
	printf("                 VFED - Versatec Remote Form Feed\n");
	printf("                 VEOT - Versatec End or Terminate\n");
	printf("                 VLTR - Versatec Remote Line Terminate\n");
	printf("                 GET_REGS - print board registers\n");
	printf("                 GET_STATUS - print formatted device status\n");
	printf("                 GET_BOARD - print board interface strapping\n");
	printf("                 GET_FLAGS - print driver internal flags\n");
	printf("                 GET_FIFO - print fifo status\n");
	printf("                 DEV_STATUS - print device status\n");
	printf("                 RDY_WAIT - wait for board and plotter ready\n");
	printf("                 HALF_WAIT - wait for fifo <1/2 full\n");
	printf("                 STREAM_ON - set Centronics data streaming mode\n");
	printf("                 STREAM_OFF - clear data streaming mode\n");
	printf("\n--More-- (enter)\n");
	fgets(dummystr, 2, stdin);
	printf("                 MASTER_CLEAR - the big hammer (use wisely)\n");
	printf("                 SOFT_ACK - issue software ack to board\n");
	printf("               * SET_CONFIG - set handshake speed and mode to <argument>\n");
	printf("               * SET_AUTOLTR - set auto line terminate count to <argument>\n");
	printf("                 setting count enables, setting to 0 disables\n");
	printf(" \n");
	printf("        -s<string> send string to plotter (in current print/plot mode)\n");
	printf("                 **** strings must be in double quotes ****\n");
	printf("        -n output carriage return/line feed pair of ascii characters (in current mode\n");
	printf("        -p output carriage return/form feed pair of ascii characters (in current mode)\n");
	printf("        -v  toggle verbose mode\n");
	printf("        -i enter interactive mode.  once in interactive mode all above options may\n");
	printf("                 be entered **one per line** w/or w/out the leading -\n");
	printf("        -h print this help info\n");
	printf("        -q exit interactive mode\n");
	printf("\n--More-- (enter)\n");
	fgets(dummystr, 2, stdin);
	printf(" \n");
	printf("Note: args are executed left to right as found, multiple files in \n");
	printf("      different modes can be sent in a single command.\n");
	printf("      e.g. ikonex -w\\\\.\\vpt1 -mPRINT -ffile1.prn -mPLOT -ffile2.plt\n");
	printf("      sends file1.prn in print mode, and then file2.plt in plot mode\n");
	printf("      commands requiring arguments (*) MUST be preceded by -a<argument>\n");
	printf(" \n");
	printf("      In interactive mode, enter one cmd/value or a<argument> per line\n");
	printf(" \n");


}

/*
 * main loop
 */

int main(int argc, char **argv)
{
	char *mode_str = NULL;
	char *file_str = NULL;
	char *cmd_str = NULL;
	char *arg_str = NULL;
	char *timeout_str = NULL;
	DWORD local_arg;
	int c;

    /*
	 * print opening banner
     */

	printf("\n%s - %s\n", IkonexBanner, VersionString);
	printf("    %s\n", TahomaTech);
	printf("    %s\n\n", ContactInfo);

	if (argc == 1) {
		useage();
		exit(-1);
	}

	interactive = 0;
	arg_set = 0;
	verbose = 0;

	/*
	* utility buffer
	*/

	local_buff_p = malloc(LOCBUFFSIZE);
	if(local_buff_p == NULL)
	{
		printf("Error allocating bufer. Exiting\n");
		exit(-1);
	}

	while (1) {
		
		/*
		 * get command line or interactive argument(s)
		 */

		c = getcommand(argc, argv, ":ihqw:m:f:b:a:c:s:t:vnp");
		if (c != EOF) {
			c = tolower(c);
			switch(c) {
			case 'q':
				printf("goodbye...\n");
				close_handle();
				exit(0);
				break;
			case 'i':
				if(verbose)
					printf("interactive mode set\n");
				interactive = TRUE;
				break;
			case 'h':
				useage();
				break;
			case 'w':
				file_str = loptarg;
				if(dev_handle != INVALID_HANDLE_VALUE)
				{
					if(verbose)
						printf("closing previously open device\n");
					close_handle();
				}
				if(verbose)
					printf("opening device = %s\n", loptarg);
				if(dev_open(loptarg))
				{
					printf("Error (main 'w'): opening device \"%s\", dev_open returns error\n", loptarg);
					try_exit();
				}
				break;

			case 'c':
				cmd_str = loptarg;
				if(verbose)
					printf("cmd_str = %s\n", cmd_str);
				if(issue_cmd(cmd_str))
					try_exit();
				arg_set = 0;
				break;

			case 'f':
				file_str = loptarg;
				if(verbose)
					printf("file_str = %s\n", file_str);
				if(send_file(file_str))
				{
					printf("Error (main 'f'): sending file - send_file returns error!\n");
					try_exit();
				}
				break;

			case 'b':
				arg_str = loptarg;
				if(verbose)
					printf("byte_str = %s\n", arg_str);
				local_buff_p[0] = (char) strtol(arg_str, (char **) NULL, 0);
				if(verbose)
					printf("data byte = 0x%x\n", local_arg);
				if(file_ptr == -1)
				{
					printf("Error (main 'b'): device not open - use -w<> to open device\n");
					try_exit();
				}

				if(dev_write(local_buff_p, 1))
				{
					printf("Error (main 'b'): dev_write returns error\n");
					try_exit();
				}
				break;

			case 'm':
				mode_str = loptarg;
				if(verbose)
					printf("mode_str = %s\n", mode_str);
				if(dev_handle == INVALID_HANDLE_VALUE)
				{
					printf("Error (main 'm'): device not open - use -w<> to open device\n");
					try_exit();
					break;
				}
				if(!strcmp(mode_str, "PLOT"))
				{
					if(verbose)
						printf("mode = PLOT\n");
					local_arg = IHCPIO_V_PLOT;
					if(dev_ioctl_out(IHCPIO_SET_VMODE, &local_arg, sizeof(DWORD)))
					{
						printf("Error (main 'm'): dev_ioctl(SET_VMODE) returns error\n");
						try_exit();
					}
				}
				else if(!strcmp(mode_str, "PRINT"))
				{
					if(verbose)
						printf("mode = PRINT\n");
					local_arg = IHCPIO_V_NPRINT;
					if(dev_ioctl_out(IHCPIO_SET_VMODE, &local_arg, sizeof(DWORD)))
					{
						printf("Error (main 'm'): dev_ioctl(SET_VMODE) returns error\n");
						try_exit();
					}
				}
				else
				{
					printf("Error (main 'm'): mode \"$s\" isnot PRINT or PLOT\n", mode_str);
					try_exit();
				}
				break;

			case 'v':
				if(verbose)
				{
					printf("verbose OFF\n");
					verbose = 0;
				}
				else
				{
					printf("verbose ON\n");
					verbose = 1;
				}
				break;

			case 'a':
				arg_str = loptarg;
				if(verbose)
					printf("arg_str = %s\n", arg_str);
				cmd_arg = (DWORD)strtol(arg_str, (char **) NULL, 0);
				if(verbose)
					printf("cmd_arg = 0x%x\n", cmd_arg);
				arg_set = 1;
				break;

			case 't':
				timeout_str = loptarg;
				if (verbose == TRUE)
					printf("timeout_string = %s\n", timeout_str);
				if(dev_handle == INVALID_HANDLE_VALUE)
				{
					printf("Error (main 't'): device not open - use -w<> to open device\n");
					try_exit();
					break;
				}
				local_arg = (DWORD)strtol(timeout_str, (char **) NULL, 0);
				if (verbose == TRUE)
					printf("timeout = %d\n", local_arg);
				if(dev_ioctl_out(IHCPIO_SET_FIFOTIME, &local_arg, sizeof(DWORD)))
				{
					printf("Error (main 't') dev_ioctl(SET_FIFOTIME) returns error\n");
					try_exit();
				}
				break;

			case 's':
				strcpy(local_buff_p, loptarg);
				if(verbose)
					printf("arg_str = %s\n", local_buff_p);
				if(dev_handle == INVALID_HANDLE_VALUE)
				{
					printf("Error (main 's') device not open - use -w<> to open device\n");
					try_exit();
					break;
				}
				if(dev_write(local_buff_p, (DWORD)strlen(local_buff_p)))
				{
					printf("Error (main 's'): sending string - dev_write returns error\n");
					try_exit();
				}
				break;

			case 'n':
				if(verbose)
					printf("sending CR/LF\n");
				if(dev_handle == INVALID_HANDLE_VALUE)
				{
					printf("Error (main 'n'): device not open - use -w<> to open device\n");
					try_exit();
					break;
				}
				strcpy(local_buff_p, "\r\n");
				if(dev_write(local_buff_p, 2))
				{
					printf("Error (main 'n'): sending CR/LF - dev_write returns error\n");
					try_exit();
				}
				break;

			case 'p':
				if(verbose)
					printf("sending CR/FF\n");
				if(dev_handle == INVALID_HANDLE_VALUE)
				{
					printf("Error (main 'p'): device not open - use -w<> to open device\n");
					try_exit();
					break;
				}
				strcpy(local_buff_p, "\r\f");
				if(dev_write(local_buff_p, 2))
				{
					printf("Error (main 'p'): sending CR/FF - dev_write returns error\n");
					try_exit();
				}
				break;

			case '?':
				printf("Error (main): unrecognized option \"%c\"\n", optopt);
				try_exit();
				break;

			case ':':
				printf("Error (main): missing parameter for option \"%c\"\n", optopt);
				try_exit();
				break;

			default:

				/*
				 * shouldn't ever get here
				 */

				printf("Error (mian): unrecognized option \"%c\"\n", c);
				try_exit();
				break;
			}
		}
		else
		{
			if (verbose)
				printf("End of arg list\n");
			break;
		}
	}
	close_handle();
	exit(0);
}

